#|
This file is part of Ordo.

Copyright (C) 2025 Ray Miller

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, version 3.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program. If not, see <https://www.gnu.org/licenses/>.
|#

(define-module (ordo playbook)
  #:use-module (ice-9 eval-string)
  #:use-module (ice-9 textual-ports)
  #:use-module (oop goops)
  #:use-module (ordo context)
  #:use-module (ordo logger)
  #:use-module (ordo play)
  #:use-module (srfi srfi-26)
  #:use-module (srfi srfi-69)
  #:export (<playbook>
            playbook
            playbook?
            playbook-name
            playbook-vars
            playbook-plays
            load-playbook
            run-playbook)
  #:re-export (play))

(define-class <playbook> ()
  (name #:init-keyword #:name #:getter playbook-name)
  (vars #:init-keyword #:vars #:getter playbook-vars)
  (plays #:init-keyword #:plays #:getter playbook-plays))

(define-method (initialize (object <playbook>) initargs)
  (next-method)
  (slot-set! object 'vars (alist->hash-table (slot-ref object 'vars))))

(define (playbook . args)
  (apply make <playbook> args))

(define (playbook? p)
  (is-a? p <playbook>))

(define (load-playbook filename)
  (log-msg 'INFO "Loading playbook " filename)
  (eval-string (call-with-input-file filename get-string-all)
               #:file filename))

(define (run-playbook pb inventory)
  (log-msg 'NOTICE "Running playbook: " (playbook-name pb))
  (parameterize ((*inventory* inventory)
                 (*playbook-vars* (playbook-vars pb)))
    (for-each run-play (playbook-plays pb))))
